// Charcoal Shader
// 
// Copyright Outerspace Software, all rights reserved

string Description = "This filter effect turns a texture into a charcoal drawing.";

float4x4	WorldViewProjection;
float4		DiffuseCol;

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state
{
   Texture = <MyTexture0>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Wrap;
   AddressV  = Wrap;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:			TEXCOORD0;
};

struct VS_OUTPUT{
    float4 position:	POSITION;
    float2 tex:			TEXCOORD0;
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;
	output.position = mul(input.position, WorldViewProjection);
	output.tex = input.tex.xy;
	return output;
}


PS_OUTPUT PS(VS_OUTPUT input){
	PS_OUTPUT output;
	
    float d=1.0/256.0;
   
    float3 rgb1 = tex2D(MyTextureSampler0, input.tex);
    float3 rgb2 = tex2D(MyTextureSampler0, input.tex + float2(d,0));
    float3 rgb3 = tex2D(MyTextureSampler0, input.tex + float2(0,d));
    
    float f = 0;
    f += abs(rgb1.x-rgb2.x);
    f += abs(rgb1.y-rgb2.y);
    f += abs(rgb1.z-rgb2.z);
    
    f += abs(rgb1.x-rgb3.x);
    f += abs(rgb1.y-rgb3.y);
    f += abs(rgb1.z-rgb3.z);
    
    f -= 0.2;  
    
    f=1-saturate(f);  
	    
    output.color = float4(f,f,f,1);
	output.color.w = DiffuseCol.w;

	return output;
}


technique Shader
{
    pass P0
    {
        Sampler[0] = (MyTextureSampler0);		
    
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    }
}
